'''
    ModernGL: PyOpenGL alternative
'''

import os

try:
    from .mgl import Error

except ImportError:

    if os.environ.get('READTHEDOCS') == 'True':
        from .mock import Error

    else:
        _IMPORT_ERROR = '\n'.join([
            'No implementation found for ModernGL',
            'Are you sure the source code is compiled properly?',
            'Hint: python3 setup.py build_ext --inplace',
        ])

        raise ImportError(_IMPORT_ERROR)

from .buffers import Buffer, BufferAccess, detect_format
from .common import (
    BLEND, CULL_FACE, DEPTH_TEST, LINE_LOOP, LINE_STRIP, LINE_STRIP_ADJACENCY, LINEAR, LINES, LINES_ADJACENCY, MIPMAP,
    NEAREST, POINTS, TRIANGLE_FAN, TRIANGLE_STRIP, TRIANGLE_STRIP_ADJACENCY, TRIANGLES, TRIANGLES_ADJACENCY, EnableFlag,
    InvalidObject, Primitive, TextureFilter
)
from .context import Context, create_context, create_standalone_context
from .framebuffers import Framebuffer
from .members import (
    Attribute, AttributeMap, ProgramStage, Subroutine, SubroutineMap, SubroutineUniform, SubroutineUniformMap, Uniform,
    UniformBlock, UniformBlockMap, UniformMap, Varying, VaryingMap
)
from .programs import ComputeShader, Program, Shader
from .renderbuffers import Renderbuffer
from .textures import Texture, Texture3D
from .vertex_arrays import VertexArray, VertexArrayAttribute

__version__ = '4.1.12'

__all__ = [
    'InvalidObject', 'EnableFlag', 'Primitive', 'Error', 'TextureFilter',
    'POINTS', 'LINES', 'LINE_STRIP', 'LINE_LOOP', 'TRIANGLES', 'TRIANGLE_STRIP', 'TRIANGLE_FAN',
    'LINES_ADJACENCY', 'LINE_STRIP_ADJACENCY', 'TRIANGLES_ADJACENCY', 'TRIANGLE_STRIP_ADJACENCY',
    'BLEND', 'DEPTH_TEST', 'CULL_FACE',
    'LINEAR', 'NEAREST', 'MIPMAP',
    'Subroutine', 'SubroutineMap', 'SubroutineUniform', 'SubroutineUniformMap',
    'Uniform', 'UniformMap', 'UniformBlock', 'UniformBlockMap',
    'Varying', 'VaryingMap', 'Attribute', 'AttributeMap',
    'Program', 'ProgramStage', 'Shader', 'ComputeShader',
    'Texture', 'Texture3D', 'Framebuffer', 'Renderbuffer',
    'VertexArray', 'VertexArrayAttribute',
    'Buffer', 'BufferAccess',
    'Context',
    'create_context', 'create_standalone_context',
    'detect_format',
]



import struct
import unittest

import ModernGL


class TestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.ctx = ModernGL.create_standalone_context()

    def tearDown(self):
        self.assertEqual(self.ctx.error, 'GL_NO_ERROR')

    @classmethod
    def tearDownClass(cls):
        cls.ctx.release()

    def test_dvec2(self):
        self.prog = self.ctx.program([
            self.ctx.vertex_shader('''
                #version 410

                in dvec2 in_a[2];
                out dvec2 out_a[2];

                void main() {
                    out_a[0] = in_a[0];
                    out_a[1] = in_a[1];
                }
            '''),
        ], ['out_a'])

        self.vbo1 = self.ctx.buffer(struct.pack('2d2d', 1.0, 1.0, 1.0, 1.0))
        self.vbo2 = self.ctx.buffer(struct.pack('2d2d', 0.0, 0.0, 0.0, 0.0))
        self.vao = self.ctx.simple_vertex_array(self.prog, self.vbo1, ['in_a'])
        self.vao.transform(self.vbo2, ModernGL.POINTS, 1)
        x1, y1, x2, y2 = struct.unpack('2d2d', self.vbo2.read())
        self.assertAlmostEqual(x1, 1.0)
        self.assertAlmostEqual(y1, 1.0)
        self.assertAlmostEqual(x2, 1.0)
        self.assertAlmostEqual(y2, 1.0)

    def test_dvec3(self):
        self.prog = self.ctx.program([
            self.ctx.vertex_shader('''
                #version 410

                in dvec3 in_a[2];
                out dvec3 out_a[2];

                void main() {
                    out_a[0] = in_a[0];
                    out_a[1] = in_a[1];
                }
            '''),
        ], ['out_a'])

        self.vbo1 = self.ctx.buffer(struct.pack('3d3d', 1.0, 1.0, 1.0, 1.0, 1.0, 1.0))
        self.vbo2 = self.ctx.buffer(struct.pack('3d3d', 0.0, 0.0, 0.0, 0.0, 0.0, 0.0))
        self.vao = self.ctx.simple_vertex_array(self.prog, self.vbo1, ['in_a'])
        self.vao.transform(self.vbo2, ModernGL.POINTS, 1)
        x1, y1, z1, x2, y2, z2 = struct.unpack('3d3d', self.vbo2.read())
        self.assertAlmostEqual(x1, 1.0)
        self.assertAlmostEqual(y1, 1.0)
        self.assertAlmostEqual(z1, 1.0)
        self.assertAlmostEqual(x2, 1.0)
        self.assertAlmostEqual(y2, 1.0)
        self.assertAlmostEqual(z2, 1.0)

if __name__ == '__main__':
    unittest.main()



'''
    ModernGL extensions main
'''

import argparse
import os
import sys
import shutil


def main(argv=None):
    '''
        main
    '''

    parser = argparse.ArgumentParser(prog='ModernGL.ext')
    group = parser.add_mutually_exclusive_group()
    group.add_argument('--list', action='store_true', help='list extensions')
    group.add_argument('--collect', action='store_true', help='collect extensions')
    args = parser.parse_args(argv)

    here = os.path.dirname(__file__)

    if args.list:
        for folder in os.listdir(here):
            if folder == '__pycache__' or not os.path.isdir(os.path.join(here, folder)):
                continue

            print('ModernGL.ext.' + folder)

    if args.collect:
        if not os.path.isfile(os.path.join(here, 'README.md')):
            print('Cannot collect extensions for the installed version of ModernGL.')
            exit()

        for path in sys.path:
            ext = os.path.normpath(os.path.join(path, 'ModernGL', 'ext'))
            if not os.path.isdir(ext):
                continue

            for folder in os.listdir(ext):
                found_ext = os.path.join(ext, folder)
                local_ext = os.path.join(here, folder)

                if folder == '__pycache__' or not os.path.isdir(found_ext) or os.path.isdir(local_ext):
                    continue

                shutil.copytree(found_ext, local_ext, ignore=lambda src, names: {'__pycache__'})
                print('copied "%s" to "%s"' % (found_ext, local_ext))

    if not args.list and not args.collect:
        parser.print_help()


if __name__ == '__main__':
    main()



'''
    ModernGL common
'''

# pylint: disable=too-few-public-methods

try:
    from . import mgl
except ImportError:
    from .mock import mgl


class InvalidObject:
    '''
        A ModernGL object turns into an InvalidObject
        once the ``release()`` method is successfully called.
    '''

    __slots__ = ['mglo']


class EnableFlag:
    '''
        EnableFlag
    '''

    __slots__ = ['flags']

    @staticmethod
    def new(flags) -> 'EnableFlag':
        '''
            For internal use only.
        '''

        res = EnableFlag.__new__(EnableFlag)
        res.flags = flags
        return res

    def __init__(self):
        self.flags = 0
        raise NotImplementedError('EnableFlag')

    def __or__(self, other) -> 'EnableFlag':
        return EnableFlag.new(self.flags | other.flags)

    def __and__(self, other) -> 'EnableFlag':
        return EnableFlag.new(self.flags & other.flags)

    def __inv__(self) -> 'EnableFlag':
        return EnableFlag.new(mgl.ENABLE_MASK & ~self.flags)

    def __repr__(self):
        return '<ModernGL.EnableFlag>'

    def __eq__(self, other):
        return self.flags == other.flags

    def __ne__(self, other):
        return self.flags != other.flags


BLEND = EnableFlag.new(mgl.BLEND)
'''
    GL_BLEND
'''

DEPTH_TEST = EnableFlag.new(mgl.DEPTH_TEST)
'''
    GL_DEPTH_TEST
'''

CULL_FACE = EnableFlag.new(mgl.CULL_FACE)
'''
    GL_CULL_FACE
'''


class Primitive:
    '''
        Primitive
    '''

    __slots__ = ['mglo', 'name']

    @staticmethod
    def new(obj, name) -> 'Primitive':
        '''
            For internal use only.
        '''

        res = Primitive.__new__(Primitive)
        obj.wrapper = res
        res.name = name
        res.mglo = obj
        return res

    def __init__(self):
        self.name = None
        self.mglo = None
        raise NotImplementedError('Primitive')

    def __repr__(self):
        return 'ModernGL.%s' % self.name

    def __eq__(self, other):
        return self.mglo is other.mglo

    def __ne__(self, other):
        return self.mglo is not other.mglo


TRIANGLES = Primitive.new(mgl.TRIANGLES, 'TRIANGLES')
'''
    GL_TRIANGLES
'''

TRIANGLE_STRIP = Primitive.new(mgl.TRIANGLE_STRIP, 'TRIANGLE_STRIP')
'''
    GL_TRIANGLE_STRIP
'''

TRIANGLE_FAN = Primitive.new(mgl.TRIANGLE_FAN, 'TRIANGLE_FAN')
'''
    GL_TRIANGLE_FAN
'''

LINES = Primitive.new(mgl.LINES, 'LINES')
'''
    GL_LINES
'''

LINE_STRIP = Primitive.new(mgl.LINE_STRIP, 'LINE_STRIP')
'''
    GL_LINE_STRIP
'''

LINE_LOOP = Primitive.new(mgl.LINE_LOOP, 'LINE_LOOP')
'''
    GL_LINE_LOOP
'''

POINTS = Primitive.new(mgl.POINTS, 'POINTS')
'''
    GL_POINTS
'''

LINE_STRIP_ADJACENCY = Primitive.new(mgl.LINE_STRIP_ADJACENCY, 'LINE_STRIP_ADJACENCY')
'''
    GL_LINE_STRIP_ADJACENCY
'''

LINES_ADJACENCY = Primitive.new(mgl.LINES_ADJACENCY, 'LINES_ADJACENCY')
'''
    GL_LINES_ADJACENCY
'''

TRIANGLE_STRIP_ADJACENCY = Primitive.new(mgl.TRIANGLE_STRIP_ADJACENCY, 'TRIANGLE_STRIP_ADJACENCY')
'''
    GL_TRIANGLE_STRIP_ADJACENCY
'''

TRIANGLES_ADJACENCY = Primitive.new(mgl.TRIANGLES_ADJACENCY, 'TRIANGLES_ADJACENCY')
'''
    GL_TRIANGLES_ADJACENCY
'''


class TextureFilter:
    '''
        TextureFilter
    '''

    __slots__ = ['mglo', 'name']

    @staticmethod
    def new(obj, name) -> 'TextureFilter':
        '''
            For internal use only.
        '''

        res = TextureFilter.__new__(TextureFilter)
        obj.wrapper = res
        res.name = name
        res.mglo = obj
        return res

    def __init__(self):
        self.name = None
        self.mglo = None
        raise NotImplementedError('TextureFilter')

    def __repr__(self):
        return 'ModernGL.%s' % self.name

    def __eq__(self, other):
        return self.mglo is other.mglo

    def __ne__(self, other):
        return self.mglo is not other.mglo


LINEAR = TextureFilter.new(mgl.LINEAR, 'LINEAR')
'''
    (GL_LINEAR, GL_LINEAR)
'''

NEAREST = TextureFilter.new(mgl.NEAREST, 'NEAREST')
'''
    (GL_NEAREST, GL_NEAREST)
'''

MIPMAP = TextureFilter.new(mgl.MIPMAP, 'MIPMAP')
'''
    (GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR)
'''



'''
    ModernGL extensions
'''

# pylint: disable=R0903, W0613

import os
import sys


class ExtensionFinder():
    '''
        ExtensionFinder
    '''

    def find_spec(self, fullname, path, target=None):
        '''
            find_spec
        '''

        prefix = 'ModernGL.ext.'

        if fullname.startswith(prefix):
            name = fullname[len(prefix):]
            if '.' in name:
                return

            here = os.path.dirname(__file__)
            path = os.path.join(here, name)

            if not os.path.isdir(path):
                local = os.path.isfile(os.path.join(here, 'README.md'))

                if local:
                    error = 'missing extension\n\nDo not run ModernGL from the git repository!'

                else:
                    error = 'missing extension\n\nHint: python3 -m pip install %s' % fullname

                raise ModuleNotFoundError(error)


sys.meta_path.append(ExtensionFinder())
